package org.nuiton.eugene.java;

import org.nuiton.eugene.models.object.ObjectModelElement;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * To manage annotations for any {@link org.nuiton.eugene.models.object.ObjectModelElement} of a classifier.
 * <p/>
 * Created: 17 déc. 2009
 *
 * @author Tony Chemit <chemit@codelutin.com> Copyright Code Lutin
 * @version $Revision: 763 $
 *          <p/>
 *          Mise a jour: $Date: 2009-12-17 02:08:01 +0100 (jeu., 17 déc. 2009) $ par :
 *          $Author: tchemit $
 * @since 2.0.0
 */
public class AnnotationsManager {

    protected Map<ObjectModelElement, List<String>> annotations;

    /**
     * Add the {@code annotation} for the given {@code element} of the classifier.
     *
     * @param element    the element where to register the annotation
     * @param annotation the annotation to register
     */
    public void addAnnotation(ObjectModelElement element, String annotation) {
        Map<ObjectModelElement, List<String>> map = getAnnotations();
        List<String> list = map.get(element);
        if (list == null) {
            list = new ArrayList<String>();
            map.put(element, list);
        }
        list.add(annotation);
    }

    /**
     * Obtain the array of annotations registred for a given element of the classifier.
     *
     * @param element the element where to search for annotations
     * @return the annotations for the element (empty arry if none found
     */
    public String[] getAnnotations(ObjectModelElement element) {
        Map<ObjectModelElement, List<String>> map = getAnnotations();
        List<String> list = map.get(element);
        return list == null ? new String[0] : list.toArray(new String[list.size()]);
    }

    protected Map<ObjectModelElement, List<String>> getAnnotations() {
        if (annotations == null) {
            annotations = new HashMap<ObjectModelElement, List<String>>();
        }
        return annotations;
    }

}
