/* *##% 
 * EUGene :: EUGene
 * Copyright (C) 2004 - 2009 CodeLutin
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Lesser Public License for more details.
 *
 * You should have received a copy of the GNU General Lesser Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/lgpl-3.0.html>.
 * ##%*/

package org.nuiton.eugene.models.object.xml;

import org.nuiton.eugene.GeneratorUtil;
import org.nuiton.eugene.models.object.ObjectModelAttribute;
import org.nuiton.eugene.models.object.ObjectModelClass;
import org.nuiton.eugene.models.object.ObjectModelClassifier;

/**
 * ObjectModelAttributeImpl.
 * 
 * Created: 14 janv. 2004
 * 
 * @author Cédric Pineau <pineau@codelutin.com> Copyright Code Lutin
 * 
 * @version $Revision: 777 $
 * 
 * Last update : $Date: 2009-12-20 17:27:20 +0100 (dim., 20 déc. 2009) $ by : $Author: tchemit $
 */
public class ObjectModelAttributeImpl extends ObjectModelParameterImpl
        implements ObjectModelAttribute {

    protected String reverseAttributeName = null;
    protected int reverseMaxMultiplicity = -1;
    protected String associationType = null;
    protected String visibility = "protected";    
    protected ObjectModelClassifier reference = null;
    protected String associationClassName = null;
    protected boolean isStatic = false;
    protected boolean isFinal = false;
    protected boolean isNavigable = true;

    public ObjectModelAttributeImpl() {
        super();
    }

    public void postInit() {
        if (name == null) {
            name = GeneratorUtil.toLowerCaseFirstLetter(GeneratorUtil
                    .getClassNameFromQualifiedName(type));
        }
        super.postInit();
    }

    public void setReverseAttributeName(String reverseAttributeName) {
        this.reverseAttributeName = reverseAttributeName;
    }

    public void setAssociationType(String associationType) {
        this.associationType = associationType;
    }

    public void setReverseMaxMultiplicity(int reverseMaxMultiplicity) {
        this.reverseMaxMultiplicity = reverseMaxMultiplicity;
    }

    public void setVisibility(String visibility) {
        this.visibility = visibility;
    }

    public void setFinal(boolean isFinal) {
        this.isFinal = isFinal;
    }

    public void setStatic(boolean isStatic) {
        this.isStatic = isStatic;
    }

    public void setNavigable(boolean isNavigable) {
        this.isNavigable = isNavigable;
    }

    /**
     * Returns whether this attribute is an aggregate or not.
     * 
     * @return a boolean indicating whether this attribute is an aggregate or
     *         not.
     */
    public boolean isAggregate() {
        return "aggregate".equals(associationType) || "shared".equals(associationType);
    }

    /**
     * Returns whether this attribute is an composite or not.
     * 
     * @return a boolean indicating whether this attribute is an composite or
     *         not.
     */
    public boolean isComposite() {
        return "composite".equals(associationType);
    }

    public String getVisibility() {
        return visibility;
    }

    public void setAssociationClassName(String associationClassName) {
        this.associationClassName = associationClassName;
    }

    /**
     * Returns whether this attribute reference a model classifier or not.
     * 
     * @return a boolean indicating whether this attribute reference a model
     *         classifier or not.
     */
    public boolean referenceClassifier() {
        return (getClassifier() != null);
    }

    /**
     * Returns the classifier referenced by this attribute or null if it does
     * not reference a model classifier.
     * 
     * @return the ObjectModelClassfifier referenced by this attribute or null
     *         if it does not reference a model classifier.
     */
    public ObjectModelClassifier getClassifier() {
        return getModel().getClassifier(type);
    }

    /**
     * Returns the attribute used to reference this class at the other end of
     * the association or null if this is not an association, or if it is not
     * bi-directionnal. @ see #getClassifier()
     * 
     * @return the ObjectModelAttribute used to reference this class at the
     *         other end of the association or null if this is not an
     *         association, or if it is not bi-directionnal.
     */
    public ObjectModelAttribute getReverseAttribute() {
        ObjectModelAttribute reverseAttribute = null;
        ObjectModelClassifier classifier = getClassifier();
        if (classifier instanceof ObjectModelClass) {
            reverseAttribute = classifier.getAttribute(getReverseAttributeName());
        }
        return reverseAttribute;
    }

    public String getReverseAttributeName() {
        if ((reverseAttributeName == null) || ("".equals(reverseAttributeName))) {
            reverseAttributeName = GeneratorUtil
                    .toLowerCaseFirstLetter(getDeclaringElement().getName());
        }
        return reverseAttributeName;
    }

    public String getName() {
        if ((name == null || "".equals(name)) && getClassifier() != null) {
            name = GeneratorUtil.toLowerCaseFirstLetter(getClassifier()
                    .getName());
        }
        return name;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.nuiton.eugene.models.object.ObjectModelAttribute#getReverseMaxMultiplicity()
     */
    public int getReverseMaxMultiplicity() {
        return reverseMaxMultiplicity;
    }

    /**
     * Returns the association class associated with this association, or null
     * if there is none.
     * 
     * @return a ObjectModelClass corresponding to the association class
     *         associated with this association, or null if there is none.
     */
    public ObjectModelClass getAssociationClass() {
        return getModel().getClass(associationClassName);
    }

    /**
     * Returns whether this association has an associated association class, ot
     * not.
     * 
     * @return a boolean indicating whether this association has an associated
     *         association class, ot not.
     */
    public boolean hasAssociationClass() {
        return (getAssociationClass() != null);
    }

    /**
     * Returns whether this attribute is static or not.
     * 
     * @return a boolean indicating whether this attribute is static or not.
     */
    public boolean isStatic() {
        return isStatic;
    }

    /**
     * Returns whether this attribute is final or not.
     * 
     * @return a boolean indicating whether this attribute is final or not.
     */
    public boolean isFinal() {
        return isFinal;
    }

    /*
     * @see org.nuiton.eugene.models.object.ObjectModelAttribute#isNavigable()
     */
    public boolean isNavigable() {
        return isNavigable;
    }

    /*
     * @see org.nuiton.eugene.models.object.ObjectModelAttribute#isOrdered()
     */
    @Override
    public boolean isOrdered() {
        return isIndexed() || super.isOrdered();
    }

    /*
     * @see org.nuiton.eugene.models.object.ObjectModelAttribute#isIndexed()
     */
    @Deprecated
    @Override
    public boolean isIndexed() {
        return hasStereotype("indexed");
    }
}
