/* *##% 
 * EUGene :: EUGene
 * Copyright (C) 2004 - 2009 CodeLutin
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Lesser Public License for more details.
 *
 * You should have received a copy of the GNU General Lesser Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/lgpl-3.0.html>.
 * ##%*/

package org.nuiton.eugene.models.state.xml;

import org.apache.commons.digester.Digester;
import org.apache.commons.digester.RuleSetBase;

/**
 * StateModelDigesterRuleSet
 * 
 * Definit principalement :
 *  - la classe d'implementation a utiliser pour chaque noeud
 *  - la methode a appeler apres chaque noeud
 * 
 * Ce jeu de regle ne cree pas l'element racine.
 * Il doit etre cree et ajoute a la pile digester avant l'appel a
 * <tt>Digester.parse(File)</tt>.
 * 
 * Exemple:
 * <code>
 *  StateModel monModel = new StateModelImpl()
 *  Digester d = new Digester();
 *  d.push(monModel);
 *  d.parse(file);
 * </code>
 * 
 * @author chatellier
 * @version $Revision: 755 $
 * 
 * Last update : $Date: 2009-12-15 00:50:27 +0100 (mar., 15 déc. 2009) $ By : $Author: tchemit $
 */
public class DigesterStateModelRuleSet extends RuleSetBase {

    protected String prefix = null;

    public DigesterStateModelRuleSet() {
        this("");
    }

    public DigesterStateModelRuleSet(String prefix) {
        super();
        this.prefix = prefix;
        this.namespaceURI = "http://www.codelutin.org/lutingenerator/stateModel";
    }

    public void addRuleInstances(Digester digester) {

        //digester.addObjectCreate("stateModel", StateModelImpl.class);
        //digester.addFactoryCreate("stateModel", DigesterStateModelFactory.class);
        // root element must be present on stask
        digester.addSetProperties("stateModel");

        digester.addObjectCreate("stateModel/stateChart",
                StateModelStateChartImpl.class);
        digester.addSetProperties("stateModel/stateChart");
        digester.addSetNext("stateModel/stateChart", "addStateChart");

        digester.addObjectCreate("*/state", StateModelSimpleStateImpl.class);
        digester.addSetProperties("*/state");
        digester.addSetNext("*/state", "addState");

        digester.addObjectCreate("*/complexeState",
                StateModelComplexeStateImpl.class);
        digester.addSetProperties("*/complexeState");
        digester.addSetNext("*/complexeState", "addState");

        digester
                .addObjectCreate("*/transition", StateModelTransitionImpl.class);
        digester.addSetProperties("*/transition");
        digester.addSetNext("*/transition", "addTransition");
    }
}
