package org.nuiton.eugene.writer;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * Contract of the chained writer engine.
 * <p/>
 * You can register inputs via the method {@link #registerInclude(String)}.
 * <p/>
 * And later obtain of the selected writers for your inputs via the method {@link #getSelectedWriters()}.
 * <p/>
 * Created: 17 déc. 2009
 *
 * @author Tony Chemit <chemit@codelutin.com> Copyright Code Lutin
 * @version $Revision: 774 $
 *          <p/>
 *          Mise a jour: $Date: 2009-12-20 14:01:03 +0100 (dim., 20 déc. 2009) $ par :
 *          $Author: tchemit $
 * @since 2.0.0
 */
public interface ChainedWriterEngine {

    ChainedFileWriterConfiguration getConfiguration();

    void setConfiguration(ChainedFileWriterConfiguration configuration);

    Set<ChainedFileWriter> getAvailableWriters();

    List<ChainedFileWriter> getSelectedWriters();

    boolean containsWriter(String inputProtocol);

    Set<ChainedFileWriter> filterWriterForModelType(Map<String, ChainedFileWriter> universe, String modelType);

    ChainedFileWriter getWriterForInputProtocol(Set<ChainedFileWriter> universe, String inputProtocol, String modelType);

    ChainedFileWriter getWriterForInclude(Set<ChainedFileWriter> universe, String include, String modelType);

    void registerInclude(String include);

    void clear();
}
